﻿using System;
using System.Collections.Generic;

namespace UMK.WzorceOperacyjne
{
    //https://sourcemaking.com/design_patterns/observer/cpp/1

    public interface IObserwator
    {
        void Aktualizuj(int dzielna);
    }

    class Dzielenie : IObserwator //obserwator
    {
        private int wartość = 0;
        private int dzielnik;

        public Dzielenie(int dzielnik)
        {
            this.dzielnik = dzielnik;
        }

        public void Aktualizuj(int dzielna)
        {
            wartość = dzielna / dzielnik;
            Console.WriteLine("{0} / {1} = {2}", dzielna, dzielnik, wartość);
        }
    }

    class Modulo : IObserwator //obserwator
    {
        private int wartość = 0;
        private int dzielnik;

        public Modulo(int dzielnik)
        {
            this.dzielnik = dzielnik;
        }

        public void Aktualizuj(int dzielna)
        {
            wartość = dzielna % dzielnik;
            Console.WriteLine("{0} % {1} = {2}", dzielna, dzielnik, wartość);
        }
    }

    /*
    class Podmiot //podmiot, obserwabla
    {
        private int wartość;
        private Dzielenie dzielenie; //obiekty, których stan zależy od zmieniającego się pola wartość
        private Modulo modulo;        

        private void powiadom()
        {
            dzielenie.Aktualizuj(wartość);
            modulo.Aktualizuj(wartość);            
        }

        public Podmiot(params IObserwator[] obserwatorzy)
        {
            wartość = 0;
            dzielenie = new Dzielenie(3);
            modulo = new Modulo(3);
        }

        public void ZmieńWartość(int nowaWartość)
        {
            wartość = nowaWartość;
            powiadom();
        }
    }
    */

    class Podmiot //podmiot, obserwabla
    {
        private int wartość;
        //private Dzielenie dzielenie; //obiekty, których stan zależy od zmieniającego się pola wartość
        //private Modulo modulo;
        private List<IObserwator> obserwatorzy = new List<IObserwator>();

        private void powiadom()
        {
            //dzielenie.Aktualizuj(wartość);
            //modulo.Aktualizuj(wartość);
            foreach (IObserwator obserwator in obserwatorzy)
                obserwator.Aktualizuj(wartość);
        }

        public Podmiot(params IObserwator[] obserwatorzy)
        {
            wartość = 0;
            //dzielenie = new Dzielenie(3);
            //modulo = new Modulo(3);
            this.obserwatorzy.AddRange(obserwatorzy);
        }

        public void PodłączObserwator(IObserwator obserwator)
        {
            if(!obserwatorzy.Contains(obserwator))
                obserwatorzy.Add(obserwator);
        }

        public void OdłączObserwatora(IObserwator obserwator)
        {
            if (obserwatorzy.Contains(obserwator))
                obserwatorzy.Remove(obserwator);
        }

        public void ZmieńWartość(int nowaWartość)
        {
            wartość = nowaWartość;
            powiadom();
        }
    }
}